@@ -20,9 +20,9 @@
 
 
 #include "fluid_defsfont.h"
+/* Todo: Get rid of that 'include' */
 #include "fluid_sys.h"
-#include "fluid_voice.h"
- 
+
 /***************************************************************
  *
  *                           SFONT LOADER
@@ -604,7 +604,7 @@ fluid_defpreset_noteon(fluid_defpreset_t* preset, fluid_synth_t* synth, int chan
       
       /* run thru all the zones of this instrument */
       inst_zone = fluid_inst_get_zone(inst);
-      while (inst_zone != NULL) {
+	  while (inst_zone != NULL) {
 	
 	/* make sure this instrument zone has a valid sample */
 	sample = fluid_inst_zone_get_sample(inst_zone);
@@ -794,7 +794,7 @@ fluid_defpreset_noteon(fluid_defpreset_t* preset, fluid_synth_t* synth, int chan
 
 	inst_zone = fluid_inst_zone_next(inst_zone);
       }
-    }
+	}
     preset_zone = fluid_preset_zone_next(preset_zone);
   }
 
@@ -1311,6 +1311,7 @@ new_fluid_inst_zone(char* name)
   }
   FLUID_STRCPY(zone->name, name);
   zone->sample = NULL;
+  zone->keylo = 0;
   zone->keyhi = 128;
   zone->vello = 0;
   zone->velhi = 128;
@@ -1380,9 +1381,10 @@ fluid_inst_zone_import_sfont(fluid_inst_zone_t* zone, SFZone *sfzone, fluid_defs
   if ((sfzone->instsamp != NULL) && (sfzone->instsamp->data != NULL)) {
     zone->sample = fluid_defsfont_get_sample(sfont, ((SFSample *) sfzone->instsamp->data)->name);
     if (zone->sample == NULL) {
-      FLUID_LOG(FLUID_ERR, "Couldnt fins sample name");
+      FLUID_LOG(FLUID_ERR, "Couldn't find sample name");
       return FLUID_FAILED;
     }
+    fluid_voice_optimize_sample(zone->sample);
   }
 
   /* Import the modulators (only SF2.1 and higher) */
