@@ -328,6 +328,8 @@ new_fluid_synth(fluid_settings_t *settings)
     return NULL;
   }
   FLUID_MEMSET(synth, 0, sizeof(fluid_synth_t));
+  
+  fluid_mutex_init(synth->busy);
 
   synth->settings = settings;
 
@@ -670,6 +672,7 @@ delete_fluid_synth(fluid_synth_t* synth)
   FLUID_FREE(synth->LADSPA_FxUnit);
 #endif
   
+  fluid_mutex_destroy(synth->busy);  
   FLUID_FREE(synth);
 
   return FLUID_OK;
@@ -695,6 +698,8 @@ fluid_synth_noteon(fluid_synth_t* synth, int chan, int key, int vel)
 {
   fluid_channel_t* channel;
   int r;
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
 
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
@@ -747,6 +752,8 @@ fluid_synth_noteoff(fluid_synth_t* synth, int chan, int key)
   int i;
   fluid_voice_t* voice;
   int status = FLUID_FAILED;
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
@@ -782,6 +789,8 @@ fluid_synth_damp_voices(fluid_synth_t* synth, int chan)
   int i;
   fluid_voice_t* voice;
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if ((voice->chan == chan) && _SUSTAINED(voice)) {
@@ -799,6 +808,8 @@ fluid_synth_damp_voices(fluid_synth_t* synth, int chan)
 int 
 fluid_synth_cc(fluid_synth_t* synth, int chan, int num, int val)
 {
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
     FLUID_LOG(FLUID_WARN, "Channel out of range");
@@ -924,6 +935,8 @@ fluid_synth_modulate_voices(fluid_synth_t* synth, int chan, int is_cc, int ctrl)
   int i;
   fluid_voice_t* voice;
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if (voice->chan == chan) {
@@ -945,6 +958,8 @@ fluid_synth_modulate_voices_all(fluid_synth_t* synth, int chan)
 {
   int i;
   fluid_voice_t* voice;
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
 
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
@@ -962,6 +977,8 @@ int
 fluid_synth_pitch_bend(fluid_synth_t* synth, int chan, int val)
 {
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
     FLUID_LOG(FLUID_WARN, "Channel out of range");
@@ -1017,6 +1034,31 @@ fluid_synth_pitch_wheel_sens(fluid_synth_t* synth, int chan, int val)
   return FLUID_OK;
 }
 
+/*
+ * fluid_synth_get_pitch_wheel_sens
+ *
+ * Note : this function was added after version 1.0 API freeze.
+ * So its API is not in the synth.h file. It should be added in some later
+ * version of fluidsynth. Maybe v2.0 ? -- Antoine Schmitt May 2003
+ */
+/*******************
+int 
+fluid_synth_get_pitch_wheel_sens(fluid_synth_t* synth, int chan, int* pval)
+{
+
+  // check the ranges of the arguments
+  if ((chan < 0) || (chan >= synth->midi_channels)) {
+    FLUID_LOG(FLUID_WARN, "Channel out of range");
+    return FLUID_FAILED;     
+  }
+
+  // get the pitch-bend value in the channel
+  *pval = synth->channel[chan]->pitch_wheel_sensitivity;
+
+  return FLUID_OK;
+}
+************/
+
 /*
  * fluid_synth_get_preset
  */
@@ -1075,6 +1117,8 @@ fluid_synth_program_change(fluid_synth_t* synth, int chan, int prognum)
   unsigned int banknum;
   unsigned int sfont_id;
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   if ((prognum >= 0) && (prognum < FLUID_NUM_PROGRAMS) &&
       (chan >= 0) && (chan < synth->midi_channels)) {
 
@@ -1298,6 +1342,8 @@ int fluid_synth_set_reverb_preset(fluid_synth_t* synth, int num)
 void fluid_synth_set_reverb(fluid_synth_t* synth, double roomsize, double damping, 
 			   double width, double level)
 {
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   fluid_revmodel_setroomsize(synth->reverb, roomsize);
   fluid_revmodel_setdamp(synth->reverb, damping);
   fluid_revmodel_setwidth(synth->reverb, width);
@@ -1310,6 +1356,8 @@ void fluid_synth_set_reverb(fluid_synth_t* synth, double roomsize, double dampin
 void fluid_synth_set_chorus(fluid_synth_t* synth, int nr, double level, 
 			   double speed, double depth_ms, int type)
 {
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
   fluid_chorus_set_nr(synth->chorus, nr);
   fluid_chorus_set_level(synth->chorus, (fluid_real_t)level);
   fluid_chorus_set_speed_Hz(synth->chorus, (fluid_real_t)speed);
@@ -1573,6 +1621,7 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
   fluid_real_t* chorus_buf;
   int byte_size = FLUID_BUFSIZE * sizeof(fluid_real_t);
   double prof_ref = fluid_profile_ref();
+  fluid_mutex_lock(synth->busy); /* Here comes the audio thread. Lock the synth. */
   
   fluid_check_fpe("??? Just starting up ???");  
 
@@ -1692,6 +1741,7 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
   {float num=1;while (num != 0){num*=0.5;};};
 #endif
   fluid_check_fpe("??? Remainder of synth_one_block ???");
+  fluid_mutex_unlock(synth->busy); /* Allow other threads to touch the synth */
 
   return 0;  
 }
@@ -1712,6 +1762,9 @@ fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
   fluid_voice_t* voice;
   int best_voice_index=-1;
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
+
   for (i = 0; i < synth->nvoice; i++) {
     
     voice = synth->voice[i];
@@ -1784,6 +1837,9 @@ fluid_synth_alloc_voice(fluid_synth_t* synth, fluid_sample_t* sample, int chan,
   int i, k;
   fluid_voice_t* voice = NULL;
 
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
+
   /* If there is another voice process on the same channel and key,
      advance it to the release phase. */
   fluid_synth_release_voice_on_same_note(synth, chan, key);
@@ -1910,12 +1966,16 @@ void fluid_synth_kill_by_exclusive_class(fluid_synth_t* synth, fluid_voice_t* ne
  */
 void fluid_synth_start_voice(fluid_synth_t* synth, fluid_voice_t* voice)
 {
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
+
   /* Find the exclusive class of this voice. If set, kill all voices
    * that match the exclusive class and are younger than the first
    * voice process created by this noteon event. */
   fluid_synth_kill_by_exclusive_class(synth, voice);  
 
   /* Start the new voice */
+
   fluid_voice_start(voice);
 }
 
@@ -2255,6 +2315,9 @@ double fluid_synth_get_reverb_width(fluid_synth_t* synth)
 void fluid_synth_release_voice_on_same_note(fluid_synth_t* synth, int chan, int key){
   int i;
   fluid_voice_t* voice;
+  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
+  fluid_mutex_unlock(synth->busy);
+
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if (_PLAYING(voice) 
