@@ -23,6 +23,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "fluidsynth_priv.h"
+
 #if !defined(WIN32) && !defined(MACINTOSH)
 #define _GNU_SOURCE
 #include <getopt.h>
@@ -34,14 +36,14 @@
 
 #include "fluidsynth.h"
 
-#ifdef WIN32
-#include "config_win32.h"
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#if defined(WIN32) && !defined(MINGW32)
+#include "config_win32.h"
+#endif
+
 #ifdef HAVE_SIGNAL_H
 #include "signal.h"
 #endif
@@ -119,18 +121,27 @@ void process_o_cmd_line_option(fluid_settings_t* settings, char* optarg){
     }
   }
   
-  /* Don't know the type of the setting in question. So try
-   * all types. The wrong types will fail. 
+  /* At this point:
+   * optarg => "synth.polyphony"
+   * val => "16"
    */
-  if (fluid_settings_setnum(settings, optarg, atof(val))){
-    printf("set %s to %s (float type)\n", optarg, val);
-  } else if (fluid_settings_setint(settings, optarg, atoi(val))){
-    printf("set %s to %s (int type)\n", optarg, val);
-  } else if (fluid_settings_setstr(settings, optarg, val)){
-    printf("set %s to %s (string type)\n", optarg, val);
-  } else {
-    printf("Failed to set %s to %s\n", optarg, val);
-  };
+  switch(fluid_settings_get_type(settings, optarg)){
+  case FLUID_NUM_TYPE:
+    if (fluid_settings_setnum(settings, optarg, atof(val))){
+      break;
+    };
+  case FLUID_INT_TYPE:
+    if (fluid_settings_setint(settings, optarg, atoi(val))){
+      break;
+    };
+  case FLUID_STR_TYPE:
+    if (fluid_settings_setstr(settings, optarg, val)){
+      break;
+    };
+  default:
+    fprintf (stderr, "Settings argument on command line: Failed to set \"%s\" to \"%s\".\n"
+	      "Most likely the parameter \"%s\" does not exist.\n", optarg, val, optarg);
+  }  
 }
 
 
@@ -235,10 +246,10 @@ int main(int argc, char** argv)
       fluid_settings_setint(settings, "audio.jack.autoconnect", 1);
       break;
     case 'z':
-      fluid_settings_setint(settings, "audio.period-size", atof(optarg));
+      fluid_settings_setint(settings, "audio.period-size", atoi(optarg));
       break;
     case 'c':
-      fluid_settings_setint(settings, "audio.periods", atof(optarg));
+      fluid_settings_setint(settings, "audio.periods", atoi(optarg));
       break;
     case 'g':
       fluid_settings_setnum(settings, "synth.gain", atof(optarg));
@@ -429,17 +440,12 @@ int main(int argc, char** argv)
     
     flags = CCA_Config_Data_Set | CCA_Terminal;
 
-#ifdef JACK_SUPPORT    
-    fluid_settings_getstr (settings, "audio.driver", &str);
-    if (str && strcmp (str, "jack") == 0)
+    if (fluid_settings_str_equal(settings, "audio.driver", "jack")) {
       flags |= CCA_Use_Jack;
-#endif /* JACK_SUPPORT */
-
-#ifdef ALSA_SUPPORT
-    fluid_settings_getstr (settings, "midi.driver", &str);
-    if (str && strcmp (str, "alsa_seq") == 0)
+    }
+    if (fluid_settings_str_equal(settings, "midi.driver", "alsa_seq")) {
       flags |= CCA_Use_Alsa;
-#endif /* ALSA_SUPPORT */
+    }
     
     fluid_cca_client = cca_init (cca_args, "FluidSynth", flags, CCA_PROTOCOL (1,1));
     
